<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is a proprietary licensed product. 
 * For more information see License.txt in the plugin folder.

 * ---
 * Copyright (c) 2019, Ebenezer Obasi
 * All rights reserved.
 * info@eobasi.com.

 * Redistribution and use in source and binary forms, with or without modification, are not permitted provided.

 * This plugin should be bought from the developer. For details contact info@eobasi.com.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * Discover search data access object.
 *
 * @author Ebenezer Obasi <eobasilive@gmail.com, info@eobasi.com>
 * @package ow.plugin.discover.bol
 * @since 1.5.3
 */

class DISCOVER_BOL_SearchDao extends OW_BaseDao
{
    /**
     * Constructor.
     *
     */
    protected function __construct()
    {
        parent::__construct();
    }
    /**
     * Singleton instance.
     *
     * @var DISCOVER_BOL_SearchDao
     */
    private static $classInstance;

    /**
     * Returns an instance of class (singleton pattern implementation).
     *
     * @return DISCOVER_BOL_SearchDao
     */
    public static function getInstance()
    {
        if ( self::$classInstance === null )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    /**
     * @see OW_BaseDao::getDtoClassName()
     *
     */
    public function getDtoClassName()
    {
        return 'BOL_User';
    }

    /**
     * @see OW_BaseDao::getTableName()
     *
     */
    public function getTableName()
    {
        return null;
    }

    /**
     * Search users matches
     *
     * @param integer $userId
     * @param integer $limit
     * @param array $excludeIds
     * @return integer
     */
    public function findUserMatches( int $userId, int $limit, array $excludeIds = [] )
    {
        $excludeIds = array_merge($excludeIds, [$userId]);
        $processedFilter = [];
        $questionsData = BOL_QuestionService::getInstance()->getQuestionData([$userId], ['sex', 'match_sex']);

        //let's try and figure out gender match for user
        //check if users are allow to filter gender
        if( (bool) OW::getConfig()->getValue('discover', 'enable_gender_filter')
        //check if gender session is set
        && OW::getSession()->isKeySet(DISCOVER_BOL_Service::SESSION_NAME))
        {
            //set sex filter
            $sex = (int) DISCOVER_BOL_Service::getInstance()->getSelectedSex();
            $processedFilter['sex'] = $sex > 0 ? [$sex] : [];
        }
        else
        {
            //check if user has match_sex question field
            if (!empty($questionsData[$userId]['match_sex'])) {
                $processedFilter['sex'] = $this->getMultipleValues($questionsData[$userId]['match_sex']);
            }
        }
        
        // we still cannot match user's sex
        if (empty($processedFilter['sex']) ||  $processedFilter['sex'] == 0){
            unset($processedFilter['sex']);
        }

        //skadate sex - match_sex compatibility
        if(isset($processedFilter['sex']) && OW::getPluginManager()->isPluginActive('skadate') )
        {
            $processedFilter['match_sex'] = $processedFilter['sex'];

            if (!empty($questionsData[$userId]['sex'])){
                $processedFilter['sex'] = $questionsData[$userId]['sex'];
            }
        }

        $processedFilter = DISCOVER_BOL_SearchService::getInstance()->updateSearchData($processedFilter);

        // add some extra conditions
        OW::getEventManager()->bind('base.query.user_filter', function(BASE_CLASS_QueryBuilderEvent $event)
        use ($processedFilter, $userId, $excludeIds)
        {
            // exclude rated users
            $event->addJoin(" LEFT JOIN `". DISCOVER_BOL_InterestDao::getInstance()->getTableName() ."` `um` ON `um`.`initiatorId` = {$userId}"
                . " AND `um`.`opponentId` = `base_user_table_alias`.`id` ");

            $event->addWhere("`um`.`id` IS NULL");

            // exclude users
            if ( count($excludeIds) )
            {
                $event->addWhere('`base_user_table_alias`.`id` NOT IN(' . implode(', ', array_map('intval', $excludeIds)) . ')');
            }
        });

        $userIdList =  DISCOVER_BOL_SearchDao::getInstance()->findUserIdListByQuestionValues($processedFilter, 0, $limit);
        $listId = 0;
        
        if (count($userIdList) > 0) {
            $listId = BOL_SearchService::getInstance()->saveSearchResult($userIdList);
        }

        return $listId;
    }

    public function findSearchResultListByLatestActivity( $userIdList, $first, $count )
    {
        if ( empty($userIdList) )
        {
            return array();
        }

        $queryParts = BOL_UserDao::getInstance()->getUserQueryFilter("user", "id",
            array(
            "method" => "DISCOVER_BOL_SearchDao::findSearchResultListByLatestActivity"
        ));

        $where = '';

		$get_moderators = BOL_AuthorizationService::getInstance()->getModeratorList();
		$moderators_Admin = array();
		foreach ( $get_moderators as $moderator )
		{
			$moderators_Admin[] = $moderator->userId;
		}	
		$sql = "SELECT `user`.* FROM `" . BOL_UserDao::getInstance()->getTableName() . "` `user`
            {$queryParts["join"]}
            WHERE `user`.`id` IN (" . $this->dbo->mergeInClause($userIdList) . ") AND `user`.`id` NOT IN (" . implode(",",$moderators_Admin) . ") $where
            ORDER BY " . (!empty($queryParts["order"]) ? $queryParts["order"] . ", " : "" ) . " `user`.`activityStamp` DESC LIMIT :from, :count ";

        return $this->dbo->queryForObjectList($sql, $this->getDtoClassName(),
                array('from' => (int) $first, 'count' => (int) $count));
    }

    public function findSearchResultListOrderedByRecentlyJoined( $userIdList, $first, $count )
    {
        if ( empty($userIdList) )
        {
            return array();
        }

        $queryParts = BOL_UserDao::getInstance()->getUserQueryFilter("user", "id",
            array(
            "method" => "DISCOVER_BOL_SearchDao::findSearchResultListOrderedByRecentlyJoined"
        ));

        $where = '';

        $sql = "SELECT `user`.* FROM `" . BOL_UserDao::getInstance()->getTableName() . "` `user`
            {$queryParts["join"]}
                
            WHERE `user`.`id` IN (" . $this->dbo->mergeInClause($userIdList) . ") $where
            ORDER BY user.joinStamp DESC, `user`.`activityStamp` DESC  LIMIT :from, :count  ";

        return $this->dbo->queryForObjectList($sql, $this->getDtoClassName(),
                array('from' => (int) $first, 'count' => (int) $count));
    }

    /**
     * Return search result item count
     *
     * @param int $listId
     * @param int $first
     * @param int $count
     * return array
     */
    public function getUserIdList( $listId, $first, $count, $excludeList = array() )
    {
        $example = new OW_Example();
        $example->andFieldEqual('searchId', (int) $listId);
        $example->setOrder(' sortOrder ');
        $example->setLimitClause($first, $count);

        if ( !empty($excludeList) )
        {
            $example->andFieldNotInArray('userId', $excludeList);
        }

        $results = BOL_SearchResultDao::getInstance()->findListByExample($example);

        $userIdList = array();

        foreach ( $results as $result )
        {
            $userIdList[] = $result->userId;
        }

        return $userIdList;
    }

    public function findUserIdListByQuestionValues( $questionValues, $first, $count, $isAdmin = false,
        $aditionalParams = array() )
    {
        $questionNameList = array_keys($questionValues);

        $questions = BOL_QuestionService::getInstance()->findQuestionByNameList($questionNameList);

        $prefix = 'qd';
        $counter = 0;
        $innerJoin = '';
        $where = '';

        foreach ( $questions as $question )
        {
            if ( !empty($questionValues[$question->name]) && $question->name != 'password' )
            {
                if ( $question->base == 1 )
                {
                    $where .= ' AND `user`.`' . $this->dbo->escapeString($question->name) . '` LIKE \'' . $this->dbo->escapeString($questionValues[$question->name]) . '%\'';
                }
                else
                {
                    $params = array(
                        'question' => $question,
                        'value' => $questionValues[$question->name],
                        'prefix' => $prefix . $counter
                    );

                    $event = new BASE_CLASS_QueryBuilderEvent("base.question.search_sql", $params);

                    OW::getEventManager()->trigger($event);

                    $data = $event->getData();

                    if ( !empty($data['join']) || !empty($data['where']) )
                    {
                        $innerJoin .= $event->getJoin();
                        $where .= ' AND ' . $event->getWhere();
                    }
                    else
                    {
                        $questionString = BOL_UserDao::getInstance()->getQuestionWhereString($question,
                            $questionValues[$question->name], $prefix . $counter);
                        if ( !empty($questionString) )
                        {
                            $innerJoin .= " INNER JOIN `" . BOL_QuestionDataDao::getInstance()->getTableName() . "` `" . $prefix . $counter . "`
                                ON ( `user`.`id` = `" . $prefix . $counter . "`.`userId` AND `" . $prefix . $counter . "`.`questionName` = '" . $this->dbo->escapeString($question->name) . "' AND " . $questionString . " ) ";
                        }
                    }

                    $counter++;
                }
            }
        }

        if ( !empty($aditionalParams['join']) )
        {
            $innerJoin .= $aditionalParams['join'];
        }

        if ( !empty($aditionalParams['where']) )
        {
            $where = $aditionalParams['where'];
        }

        if ( !empty($questionValues['accountType']) )
        {
            $where .= " AND `user`.`accountType` = '" . $this->dbo->escapeString($questionValues['accountType']) . "' ";
        }

        $queryParts = BOL_UserDao::getInstance()->getUserQueryFilter("user", "id",
            array(
            "method" => "BOL_UserDao::findUserIdListByQuestionValues"
        ));

        $order = '`user`.`activityStamp` DESC';

        if ( !empty($aditionalParams['order']) )
        {
            $order = $aditionalParams['order'];
        }

        $usersTableName = "`".BOL_UserDao::getInstance()->getTableName()."`";
        
        if ( OW_SQL_LIMIT_USERS_COUNT > 0 )
        {
            $orderFieldname = BOL_UserDao::ACTIVITY_STAMP;
            $usersTableName = "( SELECT * FROM {$usersTableName} ORDER BY `{$orderFieldname}` DESC LIMIT " . OW_SQL_LIMIT_USERS_COUNT . " )";
        }

        $query = "SELECT DISTINCT `user`.id, `user`.`activityStamp` FROM {$usersTableName} `user`
                {$innerJoin}
                {$queryParts["join"]}

                WHERE {$queryParts["where"]} {$where}
                ORDER BY {$order}
                LIMIT :first, :count ";

        if ( $isAdmin === true )
        {
            $query = "SELECT DISTINCT `user`.id FROM {$usersTableName} `user`
                {$innerJoin}
                WHERE 1 {$where}
                ORDER BY {$order}
                LIMIT :first, :count ";
        }

        return $this->dbo->queryForColumnList($query, array_merge(array('first' => $first, 'count' => $count)));
    }

    /**
     * Get multiple values
     *
     * @param integer $value
     * @return array
     */
    public function getMultipleValues($value) {
        $values = [];

        for ( $bit = 0; $bit < 32; $bit++ )
        {
            $binPower = 1 << $bit;

            if ($binPower & $value)
            {
                $values[] = $binPower;
            }
        }

        return $values;
    }
}
