<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is a proprietary licensed product. 
 * For more information see License.txt in the plugin folder.

 * ---
 * Copyright (c) 2019, Ebenezer Obasi
 * All rights reserved.
 * info@eobasi.com.

 * Redistribution and use in source and binary forms, with or without modification, are not permitted provided.

 * This plugin should be bought from the developer. For details contact info@eobasi.com.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * Discover search service class.
 *
 * @author Ebenezer Obasi <eobasilive@gmail.com, info@eobasi.com>
 * @package ow.plugin.discover.bol
 * @since 1.5.3
 */
final class DISCOVER_BOL_SearchService
{
    const LIST_ORDER_LATEST_ACTIVITY = 'latest_activity';
    const LIST_ORDER_NEW = 'new';
    const LIST_ORDER_MATCH_COMPATIBILITY = 'match_compatibility';
    const LIST_ORDER_DISTANCE = 'distanse';
    const LIST_ORDER_WITHOUT_SORT = 'without_sort';
    
    /**
     * Class instance
     *
     * @var DISCOVER_BOL_SearchService
     */
    private static $classInstance;

    private $searchDao;
    /**
     * Class constructor
     */
    private function __construct() {
        
        $this->searchDao = DISCOVER_BOL_SearchDao::getInstance();
    }

    /**
     * Returns class instance
     *
     * @return DISCOVER_BOL_Service
     */
    public static function getInstance()
    {
        if ( null === self::$classInstance )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    public function getAccounTypeByGender( $gender )
    {
        if( !OW::getPluginManager()->isPluginActive('skadate') )
        {
            return null;
        }

        $accountType2Gender = SKADATE_BOL_AccountTypeToGenderService::getInstance()->findAll();
        if ( !empty($accountType2Gender) )
        {
            foreach ( $accountType2Gender as $item )
            {
                if ( $item->genderValue == $gender )
                {
                    return $item->accountType;
                }
            }
        }

        return null;
    }

    public function getGenderByAccounType( $accountType )
    {
        if( !OW::getPluginManager()->isPluginActive('skadate') )
        {
            return null;
        }

        $accountType2Gender = SKADATE_BOL_AccountTypeToGenderService::getInstance()->findAll();
        if ( !empty($accountType2Gender) )
        {
            foreach ( $accountType2Gender as $item )
            {
                if ( $item->accountType == $accountType )
                {
                    return $item->genderValue;
                }
            }
        }

        return null;
    }

    public function updateSearchData( $data )
    {
        if ( empty($data) )
        {
            return array();
        }

        if ( isset($data['accountType']) )
        {
            unset($data['accountType']);
        }

        $accountType = null;
        
        if ( !empty($data['match_sex']) )
        {
            $accountType = $this->getAccounTypeByGender($data['match_sex']);
        }

        if ( !empty($accountType) )
        {
            $data['accountType'] = $accountType;
        }

        return $data;
    }
    
    public function getUserIdList( $listId, $first, $count, $excludeList = array() )
    {
        return $this->searchDao->getUserIdList($listId, $first, $count, $excludeList);
    }    
    
    public function getSearchResultList( $listId, $listType, $from, $count, $excludeList = array() )
    {
        if ( empty($excludeList) )
        {
            $excludeList = array();
        }
        
        if ( OW::getUser()->isAuthenticated() )
        {
            $excludeList[] = OW::getUser()->getId();
        }
        
        $userIdList = $this->getUserIdList($listId, 0, BOL_SearchService::USER_LIST_SIZE, $excludeList);
        
        if ( empty($userIdList) )
        {
            return array();
        }

        switch($listType)
        {
            case self::LIST_ORDER_NEW:
                
                return $this->searchDao->findSearchResultListOrderedByRecentlyJoined( $userIdList, $from, $count );
                
                break;
                
            case self::LIST_ORDER_MATCH_COMPATIBILITY:

                if ( OW::getPluginManager()->isPluginActive('matchmaking') && OW::getUser()->isAuthenticated() )
                {
                    $users = BOL_UserService::getInstance()->findUserListByIdList($userIdList);
                    
                    $list = array();
                    
                    foreach ( $users as $user )
                    {
                        $list[$user->id] = $user;        
                    }
                    
                    $result = MATCHMAKING_BOL_Service::getInstance()->findCompatibilityByUserIdList( OW::getUser()->getId(), $userIdList, $from, $count);
                    $usersList = array();
                    
                    foreach ( $result as $item )
                    {
                        $usersList[$item['userId']] = $list[$item['userId']];
                    }
                    
                    return $usersList;
                }
                
                break;
                
            case self::LIST_ORDER_DISTANCE:

                if ( OW::getPluginManager()->isPluginActive('googlelocation') && OW::getUser()->isAuthenticated() )
                {
                    
                    
                    $result = BOL_QuestionService::getInstance()->getQuestionData(array(OW::getUser()->getId()), array('googlemap_location'));
                    
                    if ( !empty($result[OW::getUser()->getId()]['googlemap_location']['json']) )
                    {
                        $location = $result[OW::getUser()->getId()]['googlemap_location'];
                        
                        return GOOGLELOCATION_BOL_LocationService::getInstance()->getListOrderedByDistance( $userIdList, $from, $count, $location['latitude'], $location['longitude'] );
                    }
                }
                
                break;  
                
            default:                
                $params = array(
                    'idList' => $userIdList,
                    'orderType' => $listType,
                    'from' => $from,
                    'count' => $count,
                    'userId' => OW::getUser()->isAuthenticated() ? OW::getUser()->getId() : 0
                );
                
                $event = new OW_Event('discover.get_ordered_list', $params, array());
                OW::getEventManager()->trigger($event);
                
                $data = $event->getData();
                
                if ( !empty($data) && is_array($data) )
                {
                    return $data;
                }
        }

        return $this->searchDao->findSearchResultListByLatestActivity( $userIdList, $from, $count );
    }
    
    public function getAccountTypeToQuestionList()
    {
        $accounType2QuestionList = BOL_QuestionService::getInstance()->getAccountTypesToQuestionsList();
        
        $list = array();
        /* @var $dto BOL_QuestionToAccountType */
        foreach ( $accounType2QuestionList as $dto )
        {
            $list[$dto->accountType][$dto->questionName] = $dto->questionName;
        }
        
        return $list;
    }
    
    public function findUserIdListByQuestionValues( $questionValues, $first, $count, $isAdmin = false, $aditionalParams = array() )
    {
        $first = (int) $first;
        $count = (int) $count;

        $data = array(
            'data' => $questionValues,
            'first' => $first,
            'count' => $count,
            'isAdmin' => $isAdmin,
            'aditionalParams' => $aditionalParams
        );

        $event = new OW_Event("base.question.before_user_search", $data, $data);

        OW_EventManager::getInstance()->trigger($event);

        $data = $event->getData();

        return $this->searchDao->findUserIdListByQuestionValues($data['data'], $data['first'], $data['count'], $data['isAdmin'], $data['aditionalParams']);
    }
}