<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is a proprietary licensed product. 
 * For more information see License.txt in the plugin folder.

 * ---
 * Copyright (c) 2019, Ebenezer Obasi
 * All rights reserved.
 * info@eobasi.com.

 * Redistribution and use in source and binary forms, with or without modification, are not permitted provided.

 * This plugin should be bought from the developer. For details contact info@eobasi.com.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * Discover service class.
 *
 * @author Ebenezer Obasi <eobasilive@gmail.com, info@eobasi.com>
 * @package ow.plugin.discover.bol
 * @since 1.5.3
 */

class DISCOVER_BOL_Service
{
	const USERS_COUNT = 10;

	const NOTIFY_ENTITY_TYPE_MATCH ='discover-match';
	const NOTIFY_ACTION_MATCH = 'discover-match';

	const NOTIFY_ENTITY_TYPE_SUPERLIKE = 'discover-superlike';
	const NOTIFY_ACTION_SUPERLIKE = 'discover-superlike';
	
	const SESSION_NAME = 'discover.session.selected_sex';

	const ACTION_NOPE = 'nope';
	const ACTION_LIKE = 'like';
	const ACTION_SUPERLIKE = 'superlike';

     /**
     * Class instance
     *
     * @var DISCOVER_BOL_DISCOVER_Service
     */
    private static $classInstance;

    public static function getInstance()
    {
        if ( null === self::$classInstance )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    /**
     *
     *  @var DISCOVER_BOL_SearchDao
     */
    private $searchDao;

    /**
     *
     *  @var DISCOVER_BOL_InterestDao
     */
    private $interestDao;

    /**
     *
     *  @var SKADATE_BOL_AccountTypeToGenderService
     */
    private $accountTypeToGenderService = null;

    /**
     * Class constructor
     */
    public function __construct( )
    {
        $this->searchDao = DISCOVER_BOL_SearchDao::getInstance();
        $this->interestDao = DISCOVER_BOL_InterestDao::getInstance();
		$this->userService = BOL_UserService::getInstance();
		
		if( OW::getPluginManager()->isPluginActive('skadate') )
		{
			$this->accountTypeToGenderService = SKADATE_BOL_AccountTypeToGenderService::getInstance();
		}
	}

	public function isMatch( int $userId, int $opponentId )
	{
		if( $this->userIsInterested($userId, $opponentId) && $this->userIsInterested($opponentId, $userId) )
		{
			return true;
		}

		return false;
	}

	public function createInterest(int $initiatorId, int $opponentId, $action)
	{
		$params = [
			'initiatorId' => $initiatorId,
			'opponentId' => $opponentId,
			'action' => $action,
		];

		if( $action != self::ACTION_NOPE )
		{
			$params['interestId'] = $this->saveInterest($params);
			$params['isMatch'] = (bool) $this->isMatch($initiatorId, $opponentId);
		}

		$event = new OW_Event(DISCOVER_CLASS_EventHandler::EVENT_ADD, $params);
		OW::getEventManager()->trigger($event);
	}

	public function saveInterest( array $data )
	{
		$interest = new DISCOVER_BOL_Interest();

		$interest->setInitiator($data['initiatorId']);
		$interest->setOpponent($data['opponentId']);
		$interest->setTimestamp(time());

		$this->interestDao->save($interest);

		return OW::getDbo()->getInsertId();
	}
	
	public function findUserInterests( int $userId, int $limit = self::USERS_COUNT, array $excludeIds = [] )
	{
		return $this->searchDao->findUserMatches( $userId, $limit, $excludeIds);
	}

	public function userIsInterested( int $userId, int $opponentId )
	{
		return (bool) $this->interestDao->countInterest( $userId, $opponentId );
	}
	
	public function getSelectedSex( )
	{
		if( OW::getSession()->isKeySet( self::SESSION_NAME ) )
		{
			return OW::getSession()->get( self::SESSION_NAME );
		}
		
		return 0;
	}
	
	public function setSelectedSex( $sex )
	{
		OW::getSession()->set( self::SESSION_NAME, $sex );
		return OW::getSession()->get( self::SESSION_NAME );
	}
	
	public function sexList()
	{
		$allSex = array();

		if( OW::getPluginManager()->isPluginActive('skadate') )
		{
			$sexes = $this->accountTypeToGenderService->findAll();

			foreach ( $sexes as $sex )
			{
				$allSex[$sex->genderValue]['value'] = $sex->genderValue;
				$allSex[$sex->genderValue]['name'] = OW::getLanguage()->text('base', 'questions_account_type_' . $sex->accountType);
			}
		}
		else
		{
			$sexes = BOL_QuestionValueDao::getInstance()->findQuestionValues('sex');

			foreach($sexes as $id => $sex)
			{
				$allSex[$id]['value']= $sex->value;
				$allSex[$id]['name']= OW::getLanguage()->text('base', 'questions_question_sex_value_'.$sex->value);
			}
		}
		
		return $allSex;
	}
}