<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is a proprietary licensed product. 
 * For more information see License.txt in the plugin folder.

 * ---
 * Copyright (c) 2019, Ebenezer Obasi
 * All rights reserved.
 * info@eobasi.com.

 * Redistribution and use in source and binary forms, with or without modification, are not permitted provided.

 * This plugin should be bought from the developer. For details contact info@eobasi.com.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * Discover event handler
 *
 * @author Ebenezer Obasi <eobasilive@gmail.com, info@eobasi.com>
 * @package ow.plugin.discover.classes
 * @since 1.5.3
 */

 class DISCOVER_CLASS_EventHandler
 {
	const EVENT_LOAD_LIMIT = 'discover.load_limit';
	const EVENT_ADD = 'discover.add_interest';
	const EVENT_USER_ID_LIST = 'discover.get_user_id_list';

	private function __construct( )
	{

	}

	public static $classInstance;

	public static function getInstance()
	{
		if( self::$classInstance === null )
		{
			self::$classInstance = new self();
		}
		return self::$classInstance;
	}

	public function init()
	{
		OW::getEventManager()->bind('notifications.collect_actions', array($this, 'onNotifyActions'));
		OW::getEventManager()->bind('admin.add_auth_labels', array($this, 'addAuthLabels'));
		
		OW::getEventManager()->bind(self::EVENT_LOAD_LIMIT, array($this, 'loadLimit'));
		OW::getEventManager()->bind(self::EVENT_ADD, array($this, 'onAddInterest'));
	}

	public function loadLimit( OW_Event $e )
	{
		$e->setData(10);
	}

	public function onAddInterest( OW_Event $e )
    {
        $params = $e->getParams();

		if( empty( $initiatorId = (int) $params['initiatorId'])
		|| empty( $opponentId = (int) $params['opponentId'] ) )
			return;

		$initiatorDisplayName = BOL_UserService::getInstance()->getDisplayName( $initiatorId );
		$opponentDisplayName = BOL_UserService::getInstance()->getDisplayName( $opponentId );
		$initiatorUrl = BOL_UserService::getInstance()->getUserUrl( $initiatorId );

		$avatars = BOL_AvatarService::getInstance()->getDataForUserAvatars(array($initiatorId));

		switch( $params['action'] )
		{
			case DISCOVER_BOL_Service::ACTION_SUPERLIKE:
				$event = new OW_Event('notifications.add', [
					'pluginKey' => 'discover',
					'entityType' => DISCOVER_BOL_Service::NOTIFY_ENTITY_TYPE_SUPERLIKE,
					'action' => DISCOVER_BOL_Service::NOTIFY_ACTION_SUPERLIKE,
					'entityId' => $params['interestId'],
					'userId' => $opponentId,
					'time' => time()
				], [
					'avatar' => $avatars[$initiatorId],
					'string' => [
						'key' => 'discover+notification_superlike_text',
						'vars' => [
							'userName' => $initiatorDisplayName,
							'url' => $initiatorUrl
						]
					],
					'url' => $initiatorUrl
				]);
				OW::getEventManager()->trigger($event);

			case DISCOVER_BOL_Service::ACTION_LIKE:
			case DISCOVER_BOL_Service::ACTION_SUPERLIKE: if( TRUE === (bool) $params['isMatch'] ):
				$event = new OW_Event('notifications.add', [
					'pluginKey' => 'discover',
					'entityType' => DISCOVER_BOL_Service::NOTIFY_ENTITY_TYPE_MATCH,
					'action' => DISCOVER_BOL_Service::NOTIFY_ACTION_MATCH,
					'entityId' => $params['interestId'],
					'userId' => $opponentId,
					'time' => time()
				], [
					'avatar' => $avatars[$initiatorId],
					'string' => [
						'key' => 'discover+notification_match_text',
						'vars' => [
							'userName' => $initiatorDisplayName,
							'url' => $initiatorUrl
						]
					],
					'url' => $initiatorUrl
				]);
				OW::getEventManager()->trigger($event);
				
				if( OW::getPluginManager()->isPluginActive('mailbox') )
				{
					$conversationService = MAILBOX_BOL_ConversationService::getInstance();
					$text = OW::getLanguage()->text('discover', 'message_text', [
						'initiator' => $initiatorDisplayName,
						'opponent' => $opponentDisplayName,
					]);
					$conversation = $conversationService->createChatConversation( $initiatorId, $opponentId );
					$conversationService->createMessage( $conversation, $initiatorId, $text );
				}
			endif;
			break;
			case DISCOVER_BOL_Service::ACTION_NOPE:
			default:
				return;
		}
    }

	public function onNotifyActions( BASE_CLASS_EventCollector $event )
	{

		$event->add([
			'section' => 'discover',
			'sectionIcon' => 'ow_ic_favorite',
			'action' => DISCOVER_BOL_Service::NOTIFY_ACTION_SUPERLIKE,
			'sectionLabel' => OW::getLanguage()->text('discover', 'notification_section_label'),
			'description' => OW::getLanguage()->text('discover', 'email_notifications_setting_superlike'),
			'selected' => true,
		]);

		$event->add([
			'section' => 'discover',
			'action' => DISCOVER_BOL_Service::NOTIFY_ACTION_MATCH,
			'description' => OW::getLanguage()->text('discover', 'email_notifications_setting_interest'),
			'selected' => true,
			'sectionLabel' => OW::getLanguage()->text('discover', 'notification_section_label'),
			'sectionIcon' => 'ow_ic_users'
		]);

	}

	public function addAuthLabels( BASE_CLASS_EventCollector $event )
    {
        $event->add([
			'discover'=> [
				'label' => OW::getLanguage()->text('discover', 'auth_group_label'),
				'actions' => [
					'play_game' => OW::getLanguage()->text('discover', 'auth_action_label_play_game'),
					'play_game_superlike' => OW::getLanguage()->text('discover', 'auth_action_label_play_game_superlike')
				]
			]
		] );
    }
 }