<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is a proprietary licensed product. 
 * For more information see License.txt in the plugin folder.

 * ---
 * Copyright (c) 2019, Ebenezer Obasi
 * All rights reserved.
 * info@eobasi.com.

 * Redistribution and use in source and binary forms, with or without modification, are not permitted provided.

 * This plugin should be bought from the developer. For details contact info@eobasi.com.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * User card game.
 *
 * @author Ebenezer Obasi <eobasilive@gmail.com, info@eobasi.com>
 * @package ow.plugin.discover.components
 * @since 1.5.3
 */

 class DISCOVER_CMP_Game extends OW_Component
 {
    protected $userId;
    protected $cmpId;

    /**
     * @return Constructor.
     */
    public function __construct( $params )
    {
        parent::__construct();

		if ( !OW::getUser()->isAuthenticated() || !OW::getUser()->isAuthorized('discover', 'play_game') )
		{
			$this->setVisible(false);
			return;
        }
        
        $params = array_merge([
            'hide_gender_filter' => false,
            'hide_panel_control' => false,
            'hide_tips' => false,
        ], $params);

        if( (bool) OW::getConfig()->getValue('discover', 'enable_gender_filter') === FALSE )
            $params['hide_gender_filter'] = true;

		$this->userId = OW::getUser()->getId();
		$this->cmpId = rand(100, 100000);

        $avatarService = BOL_AvatarService::getInstance();
		$service = DISCOVER_BOL_Service::getInstance();
        
        $defaultAvatar = $avatarService->getDefaultAvatarUrl(2);
        $userAvatar = $avatarService->userHasAvatar( $this->userId ) ? $avatarService->getAvatarUrl( $this->userId, 2): $defaultAvatar;
				
		$this->assign('allSex', $service->sexList());
		$this->assign('selectedSex', $service->getSelectedSex() );

        $this->assign('userAvatar', $userAvatar);
        $this->assign('settings', $params);
        $this->assign('imgDir', OW::getPluginManager()->getPlugin('discover')->getStaticUrl().'images/');
    }
    
    public function onBeforeRender( )
    {
        parent::onBeforeRender();

        $plugin = OW::getPluginManager()->getPlugin('discover');
		$document = OW::getDocument();

		OW::getLanguage()->addKeyForJs('discover', 'no_users');
		OW::getLanguage()->addKeyForJs('discover', 'hide');
		OW::getLanguage()->addKeyForJs('discover', 'show');
		OW::getLanguage()->addKeyForJs('discover', 'interest_indicated');
		OW::getLanguage()->addKeyForJs('discover', 'superlike_permission');

        $jsParamsArray = array(
            'cmpId' => $this->cmpId,
            'userId' => $this->userId,
            'ajaxResponder' => OW::getRouter()->urlFor('DISCOVER_CTRL_Ajax', 'ajaxResponder')
        );

		$jsGenerator = UTIL_JsGenerator::newInstance();
		$jsGenerator->setVariable('window.OW_DiscoverParams', json_encode($jsParamsArray));

		$document->addScriptDeclaration($jsGenerator->generateJs());
		$document->addStyleSheet( $plugin->getStaticCssUrl(). 'animate.min.css' );
		$document->addStyleSheet( $plugin->getStaticCssUrl(). 'style.min.css' );
        $document->addScript($plugin->getStaticJsUrl() . 'bundle.min.js');
        $document->addScript($plugin->getStaticJsUrl() . 'script.min.js');
    }
 }