<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is a proprietary licensed product. 
 * For more information see License.txt in the plugin folder.

 * ---
 * Copyright (c) 2019, Ebenezer Obasi
 * All rights reserved.
 * info@eobasi.com.

 * Redistribution and use in source and binary forms, with or without modification, are not permitted provided.

 * This plugin should be bought from the developer. For details contact info@eobasi.com.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * Discover game widget.
 *
 * @author Ebenezer Obasi <eobasilive@gmail.com, info@eobasi.com>
 * @package ow.plugin.discover.components
 * @since 1.5.3
 */

 class DISCOVER_CMP_Widget extends BASE_CLASS_Widget
 {
    /**
     * @return Constructor.
     */
    public function __construct( BASE_CLASS_WidgetParameter $params )
    {
        parent::__construct();

		if ( !OW::getUser()->isAuthenticated() || !OW::getUser()->isAuthorized('discover', 'play_game') )
		{
			$this->setVisible(false);
			return;
        }

		$this->addComponent('cmp', new DISCOVER_CMP_Game($params->customParamList));
    }
    
    public static function getSettingList()
    {
        $lang = OW::getLanguage();
        $settingList = [];

        $settingList['hide_gender_filter'] = [
            'presentation' => self::PRESENTATION_CHECKBOX,
            'label' => $lang->text('discover', 'label_hide_gender_filter'),
            'value' => '0'
        ];

        $settingList['hide_panel_control'] = [
            'presentation' => self::PRESENTATION_CHECKBOX,
            'label' => $lang->text('discover', 'label_hide_panel_control'),
            'value' => '0'
        ];

        $settingList['hide_tips'] = [
            'presentation' => self::PRESENTATION_CHECKBOX,
            'label' => $lang->text('discover', 'label_hide_tips'),
            'value' => '0'
        ];

        return  $settingList;
    }

    public static function getStandardSettingValueList()
    {
        return array(
            self::SETTING_SHOW_TITLE => true,
            self::SETTING_ICON => self::ICON_HEART,
            self::SETTING_TITLE => OW::getLanguage()->text('discover', 'cmp_widget_title'),
            self::SETTING_WRAP_IN_BOX => true
        );
    }

    public static function getAccess()
    {
        return self::ACCESS_MEMBER;
    }
 }