<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is a proprietary licensed product. 
 * For more information see License.txt in the plugin folder.

 * ---
 * Copyright (c) 2019, Ebenezer Obasi
 * All rights reserved.
 * info@eobasi.com.

 * Redistribution and use in source and binary forms, with or without modification, are not permitted provided.

 * This plugin should be bought from the developer. For details contact info@eobasi.com.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * Ajax controller.
 *
 * @author Ebenezer Obasi <eobasilive@gmail.com, info@eobasi.com>
 * @package ow.plugin.discover.controllers
 * @since 1.5.3
 */

 class DISCOVER_CTRL_Ajax extends OW_ActionController
 {
	public function __construct( )
	{
		parent::__construct();

		 if ( !OW::getRequest()->isAjax() )
        {
            throw new Redirect403Exception();
        }

        if ( !OW::getUser()->isAuthenticated() )
        {
            throw new AuthenticateException();
        }

		if ( !OW::getUser()->isAuthorized('discover', 'play_game') )
		{
			$status = BOL_AuthorizationService::getInstance()->getActionStatus('discover', 'play_game');

			echo json_encode(
				array(
					'error' => $status['msg']
				)
			);

			exit;
		}
	}

    public function ajaxResponder( $params )
    {
        $_DATA = $_SERVER['REQUEST_METHOD'] == 'POST' ? $_POST : $_GET;

        if ( isset($_DATA['ajaxFunc']) )
        {
			$callFunc = (string)$_DATA['ajaxFunc'];
            $result = call_user_func(array($this, $callFunc), $_DATA);
        }
        else
        {
            throw new Redirect404Exception();
        }

        header('Content-Type: application/json');
        exit(json_encode($result));
	}

	public function createInterest( $params )
	{
		$opponentId = $params['opponentId'];
        $action = $params['action'];

        $response = [];

        if( !empty($opponentId) && !empty($user = BOL_UserService::getInstance()->findUserById($opponentId)) && ($userId = OW::getUser()->getId()) )
        {
            switch( $action )
            {
                case DISCOVER_BOL_Service::ACTION_SUPERLIKE:
                if( !OW::getUser()->isAuthorized('discover', 'play_game_superlike') )
                {
                    $status = BOL_AuthorizationService::getInstance()->getActionStatus('discover', 'search_site');
                    return $status;
                }
                break;
            }

            DISCOVER_BOL_Service::getInstance()->createInterest($userId, $user->id, $action);
        }

        return $response;
	}
	
	public function saveGender( $params )
	{
		DISCOVER_BOL_Service::getInstance()->setSelectedSex( $params['sex'] );
		exit();
	}
	
	public function findUserMatches( $params )
	{
		$response = ['status' => 'fail', 'list' => [], 'ids' => []];

        if ( OW::getUser()->isAuthenticated() && OW::getUser()->isAuthorized('base', 'search_users') ) 
        {
            $loggedUserId = OW::getUser()->getId();
			$excludeIds = !empty($params['exclude']) ? explode(',', $params['exclude']) : [];
			
			$event = new OW_Event(DISCOVER_CLASS_EventHandler::EVENT_LOAD_LIMIT, array());
			OW::getEventManager()->trigger($event);
			$card_limit = $event->getData();

            $response['status'] = 'ok';

			$searchId = DISCOVER_BOL_Service::getInstance()->findUserInterests($loggedUserId, $card_limit, $excludeIds);

            if ( $searchId )
            {
                $foundUsers = DISCOVER_BOL_SearchService::getInstance()->
					getSearchResultList($searchId, DISCOVER_BOL_SearchService::LIST_ORDER_NEW, 0, $card_limit);

                $ids = [];
                $processedUsers = [];
 
                // process users
                if ( $foundUsers )
                {
                    foreach ( $foundUsers as $userDto )
                    {
                        $ids[] = (int) $userDto->id;

                        $processedUsers[$userDto->id] = [
                            'id' => (int) $userDto->id,
                            'userName' => $userDto->username,
                            'type' => $userDto->accountType,
                            'aboutMe' => null,
                            'age' => null,
                            'isOnline' => false,
                            'compatibility' => 0,
                            'distance' => null,
                            'url' => null,
                            'avatar' => null
                        ];
                    }

                    // find avatar
                    $avatarList = BOL_AvatarService::getInstance()->getAvatarsUrlList($ids, 2);

                    foreach( $avatarList as $userId => $avatar ) 
                    {
                        $processedUsers[$userId]['avatar'] = $avatar;
                    }

                    // find urls
                    $urls = BOL_UserService::getInstance()->getUserUrlsForList($ids);

                    foreach( $urls as $userId => $url ) 
                    {
                        $processedUsers[$userId]['url'] = $url;
                    }

                    // find compatibility
                    if ( OW::getPluginManager()->isPluginActive('matchmaking') ) 
                    {
                        $result = MATCHMAKING_BOL_Service::getInstance()->
                                findCompatibilityByUserIdList($loggedUserId, $ids, 0, count($ids));

                        foreach ($result as $item) {
                            $processedUsers[$item['userId']]['compatibility'] = (int) $item['compatibility'];
                        }
                    }

                    // find online statuses
                    $onlineStatuses = BOL_UserService::getInstance()->findOnlineStatusForUserList($ids);

                    foreach( $onlineStatuses as $userId => $isOnline ) 
                    {
                        $processedUsers[$userId]['isOnline'] = (bool) $isOnline;
                    }

                    // find display names
                    $displayNames = BOL_UserService::getInstance()->getDisplayNamesForList($ids);

                    foreach( $displayNames as $userId => $displayName ) 
                    {
                        if ($displayName) {
                            $processedUsers[$userId]['userName'] = $displayName;
                        }
                    }

                    // find some questions
                    $questionList = BOL_QuestionService::getInstance()->getQuestionData($ids, ['birthdate', 'aboutme']);

                    foreach ( $questionList as $userId => $questions ) 
                    {
                        if (isset($questions['birthdate'])) {
                            $date = UTIL_DateTime::parseDate($questions['birthdate'], UTIL_DateTime::MYSQL_DATETIME_DATE_FORMAT);
                            $processedUsers[$userId]['age'] = UTIL_DateTime::getAge($date['year'], $date['month'], $date['day']);
                        }

                        $processedUsers[$userId]['aboutMe'] = isset($questions['aboutme']) ? $questions['aboutme'] : null;
                    }
                }

                foreach( $processedUsers as $user )
                {
                    $response['list'][] = OW::getClassInstance('DISCOVER_CMP_UserCard', $user)->render();
                }

                $response['ids'] = $ids;
            }
		}
		
        return(json_encode($response));
	}

	private function getResult( $type, $langKey, $exit = true)
	{
		echo json_encode( array( $type => OW::getLanguage()->text('discover', $langKey) ) );
		if( $exit ) exit;
	}
 }